@@ -1,25 +1,24 @@
 """HTTP Client for asyncio."""
 
-__all__ = ['request']
-
 import asyncio
 import http.cookies
 import io
 import json
 import mimetypes
+import os
 import urllib.parse
 import weakref
 import warnings
-try:
-    import chardet
-except ImportError:  # pragma: no cover
-    chardet = None
+import chardet
 
 import aiohttp
 from . import hdrs, helpers, streams
 from .log import client_logger
 from .streams import EOF_MARKER, FlowControlStreamReader
-from .multidict import CIMultiDictProxy, MultiDictProxy, MultiDict
+from .multidict import CIMultiDictProxy, MultiDictProxy, MultiDict, CIMultiDict
+from .multipart import MultipartWriter
+
+__all__ = ('request', 'ClientSession')
 
 HTTP_PORT = 80
 HTTPS_PORT = 443
@@ -68,7 +67,7 @@ def request(method, url, *,
     :type chunked: bool or int
     :param bool expect100: Expect 100-continue response from server.
     :param connector: BaseConnector sub-class instance to support
-       connection pooling and session cookies.
+       connection pooling.
     :type connector: aiohttp.connector.BaseConnector
     :param bool read_until_eof: Read response until eof if response
        does not have Content-Length header.
@@ -85,76 +84,236 @@ def request(method, url, *,
       >>> data = yield from resp.read()
 
     """
-    redirects = 0
-    method = method.upper()
-    if loop is None:
-        loop = asyncio.get_event_loop()
-    if request_class is None:
-        request_class = ClientRequest
-    if connector is None:
-        connector = aiohttp.TCPConnector(force_close=True, loop=loop)
-
-    while True:
-        req = request_class(
-            method, url, params=params, headers=headers, data=data,
-            cookies=cookies, files=files, encoding=encoding,
-            auth=auth, version=version, compress=compress, chunked=chunked,
-            loop=loop, expect100=expect100, response_class=response_class)
-
-        conn = yield from connector.connect(req)
-        try:
-            resp = req.send(conn.writer, conn.reader)
-            try:
-                yield from resp.start(conn, read_until_eof)
-            except:
-                resp.close()
-                conn.close()
-                raise
-        except (aiohttp.HttpProcessingError,
-                aiohttp.ServerDisconnectedError) as exc:
-            raise aiohttp.ClientResponseError() from exc
-        except OSError as exc:
-            raise aiohttp.ClientOSError() from exc
-
-        # redirects
-        if resp.status in (301, 302, 303, 307) and allow_redirects:
-            redirects += 1
-            if max_redirects and redirects >= max_redirects:
-                resp.close(force=True)
-                break
-
-            # For 301 and 302, mimic IE behaviour, now changed in RFC.
-            # Details: https://github.com/kennethreitz/requests/pull/269
-            if resp.status != 307:
-                method = 'GET'
-                data = None
-            cookies = resp.cookies
-
-            r_url = (resp.headers.get(hdrs.LOCATION) or
-                     resp.headers.get(hdrs.URI))
-
-            scheme = urllib.parse.urlsplit(r_url)[0]
-            if scheme not in ('http', 'https', ''):
-                resp.close(force=True)
-                raise ValueError('Can redirect only to http or https')
-            elif not scheme:
-                r_url = urllib.parse.urljoin(url, r_url)
-
-            url = urllib.parse.urldefrag(r_url)[0]
-            if url:
-                yield from asyncio.async(resp.release(), loop=loop)
-                continue
-
-        break
-
+    session = ClientSession(connector=connector, loop=loop,
+                            request_class=request_class,
+                            response_class=response_class,
+                            cookies=cookies)
+    resp = yield from session.request(method, url,
+                                      params=params,
+                                      data=data,
+                                      headers=headers,
+                                      files=files,
+                                      auth=auth,
+                                      allow_redirects=allow_redirects,
+                                      max_redirects=max_redirects,
+                                      encoding=encoding,
+                                      version=version,
+                                      compress=compress,
+                                      chunked=chunked,
+                                      expect100=expect100,
+                                      read_until_eof=read_until_eof)
     return resp
 
 
+class ClientSession:
+
+    def __init__(self, *, connector=None, loop=None, request_class=None,
+                 response_class=None, cookies=None, headers=None, auth=None):
+        if loop is None:
+            loop = asyncio.get_event_loop()
+        self._loop = loop
+        self.cookies = http.cookies.SimpleCookie()
+        if connector is None:
+            connector = aiohttp.TCPConnector(force_close=True, loop=loop)
+        # For Backward compatability with `share_cookie` connectors
+        elif connector._share_cookies:
+            self._update_cookies(connector.cookies)
+        if cookies is not None:
+            self._update_cookies(cookies)
+        self._connector = connector
+        self._default_auth = auth
+
+        # Convert to list of tuples
+        if headers:
+            if isinstance(headers, dict):
+                headers = list(headers.items())
+            elif isinstance(headers, (MultiDictProxy, MultiDict)):
+                headers = list(headers.items())
+        self._default_headers = headers
+
+        if request_class is None:
+            request_class = ClientRequest
+        self._request_class = request_class
+        self._response_class = response_class
+
+    @asyncio.coroutine
+    def request(self, method, url, *,
+                params=None,
+                data=None,
+                headers=None,
+                files=None,
+                auth=None,
+                allow_redirects=True,
+                max_redirects=10,
+                encoding='utf-8',
+                version=aiohttp.HttpVersion11,
+                compress=None,
+                chunked=None,
+                expect100=False,
+                read_until_eof=True):
+
+        redirects = 0
+        method = method.upper()
+
+        # Merge with default headers and transform to CIMultiDict
+        headers = self._prepare_headers(headers)
+        if auth is None:
+            auth = self._default_auth
+        # It would be confusing if we support explicit Authorization header
+        # with `auth` argument
+        if (headers is not None and
+                auth is not None and
+                hdrs.AUTHORIZATION in headers):
+            raise ValueError("Can't combine `Authorization` header with "
+                             "`auth` argument")
+
+        while True:
+            req = self._request_class(
+                method, url, params=params, headers=headers, data=data,
+                cookies=self.cookies, files=files, encoding=encoding,
+                auth=auth, version=version, compress=compress, chunked=chunked,
+                expect100=expect100,
+                loop=self._loop, response_class=self._response_class)
+
+            conn = yield from self._connector.connect(req)
+            try:
+                resp = req.send(conn.writer, conn.reader)
+                try:
+                    yield from resp.start(conn, read_until_eof)
+                except:
+                    resp.close()
+                    conn.close()
+                    raise
+            except (aiohttp.HttpProcessingError,
+                    aiohttp.ServerDisconnectedError) as exc:
+                raise aiohttp.ClientResponseError() from exc
+            except OSError as exc:
+                raise aiohttp.ClientOSError() from exc
+
+            self._update_cookies(resp.cookies)
+            # For Backward compatability with `share_cookie` connectors
+            if self._connector._share_cookies:
+                self._connector.update_cookies(resp.cookies)
+
+            # redirects
+            if resp.status in (301, 302, 303, 307) and allow_redirects:
+                redirects += 1
+                if max_redirects and redirects >= max_redirects:
+                    resp.close(force=True)
+                    break
+
+                # For 301 and 302, mimic IE behaviour, now changed in RFC.
+                # Details: https://github.com/kennethreitz/requests/pull/269
+                if resp.status != 307:
+                    method = hdrs.METH_GET
+                    data = None
+
+                r_url = (resp.headers.get(hdrs.LOCATION) or
+                         resp.headers.get(hdrs.URI))
+
+                scheme = urllib.parse.urlsplit(r_url)[0]
+                if scheme not in ('http', 'https', ''):
+                    resp.close(force=True)
+                    raise ValueError('Can redirect only to http or https')
+                elif not scheme:
+                    r_url = urllib.parse.urljoin(url, r_url)
+
+                url = urllib.parse.urldefrag(r_url)[0]
+                if url:
+                    yield from asyncio.async(resp.release(), loop=self._loop)
+                    continue
+
+            break
+
+        return resp
+
+    def _update_cookies(self, cookies):
+        """Update shared cookies."""
+        if isinstance(cookies, dict):
+            cookies = cookies.items()
+
+        for name, value in cookies:
+            if isinstance(value, http.cookies.Morsel):
+                # use dict method because SimpleCookie class modifies value
+                # before Python3.4
+                dict.__setitem__(self.cookies, name, value)
+            else:
+                self.cookies[name] = value
+
+    def _prepare_headers(self, headers):
+        """ Add default headers and transform it to CIMultiDict
+        """
+        # Convert headers to MultiDict
+        result = CIMultiDict()
+        if headers:
+            if isinstance(headers, dict):
+                headers = headers.items()
+            elif isinstance(headers, (MultiDictProxy, MultiDict)):
+                headers = headers.items()
+            for key, value in headers:
+                result.add(key, value)
+        # Add defaults only if those are not overridden
+        if self._default_headers:
+            for key, value in self._default_headers:
+                if key not in result:
+                    result.add(key, value)
+        return result
+
+    @asyncio.coroutine
+    def get(self, url, *, allow_redirects=True, **kwargs):
+        resp = yield from self.request(hdrs.METH_GET, url,
+                                       allow_redirects=allow_redirects,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def options(self, url, *, allow_redirects=True, **kwargs):
+        resp = yield from self.request(hdrs.METH_OPTIONS, url,
+                                       allow_redirects=allow_redirects,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def head(self, url, *, allow_redirects=False, **kwargs):
+        resp = yield from self.request(hdrs.METH_HEAD, url,
+                                       allow_redirects=allow_redirects,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def post(self, url, *, data=None, **kwargs):
+        resp = yield from self.request(hdrs.METH_POST, url,
+                                       data=data,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def put(self, url, *, data=None, **kwargs):
+        resp = yield from self.request(hdrs.METH_PUT, url,
+                                       data=data,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def patch(self, url, *, data=None, **kwargs):
+        resp = yield from self.request(hdrs.METH_PATCH, url,
+                                       data=data,
+                                       **kwargs)
+        return resp
+
+    @asyncio.coroutine
+    def delete(self, url, **kwargs):
+        resp = yield from self.request(hdrs.METH_DELETE, url,
+                                       **kwargs)
+        return resp
+
+
 class ClientRequest:
 
-    GET_METHODS = {'GET', 'HEAD', 'OPTIONS'}
-    POST_METHODS = {'PATCH', 'POST', 'PUT', 'TRACE', 'DELETE'}
-    ALL_METHODS = GET_METHODS.union(POST_METHODS)
+    GET_METHODS = {hdrs.METH_GET, hdrs.METH_HEAD, hdrs.METH_OPTIONS}
+    POST_METHODS = {hdrs.METH_PATCH, hdrs.METH_POST, hdrs.METH_PUT}
+    ALL_METHODS = GET_METHODS.union(POST_METHODS).union(
+        {hdrs.METH_DELETE, hdrs.METH_TRACE})
 
     DEFAULT_HEADERS = {
         hdrs.ACCEPT: '*/*',
@@ -291,7 +450,7 @@ class ClientRequest:
 
     def update_headers(self, headers):
         """Update request headers."""
-        self.headers = MultiDict()
+        self.headers = CIMultiDict()
         if headers:
             if isinstance(headers, dict):
                 headers = headers.items()
@@ -299,7 +458,7 @@ class ClientRequest:
                 headers = headers.items()
 
             for key, value in headers:
-                self.headers.add(key.upper(), value)
+                self.headers.add(key, value)
 
         for hdr, val in self.DEFAULT_HEADERS.items():
             if hdr not in self.headers:
@@ -384,7 +543,13 @@ class ClientRequest:
             assert not isinstance(data, io.StringIO), \
                 'attempt to send text data instead of binary'
             self.body = data
-            self.chunked = True
+            if not self.chunked and isinstance(data, io.BufferedReader):
+                # Not chunking if content-length can be determined
+                size = os.fstat(data.fileno()).st_size - data.tell()
+                self.headers[hdrs.CONTENT_LENGTH] = str(size)
+                self.chunked = False
+            else:
+                self.chunked = True
             if hasattr(data, 'mode'):
                 if data.mode == 'r':
                     raise ValueError('file {!r} should be open in binary mode'
@@ -395,6 +560,11 @@ class ClientRequest:
                 mime = 'application/octet-stream' if mime is None else mime
                 self.headers[hdrs.CONTENT_TYPE] = mime
 
+        elif isinstance(data, MultipartWriter):
+            self.body = data.serialize()
+            self.headers.update(data.headers)
+            self.chunked = self.chunked or 8192
+
         else:
             if not isinstance(data, helpers.FormData):
                 data = helpers.FormData(data)
@@ -460,7 +630,7 @@ class ClientRequest:
                             result = stream.send(value)
                     except StopIteration as exc:
                         if isinstance(exc.value, bytes):
-                            yield from request.write(exc.value)
+                            yield from request.write(exc.value, drain=True)
                         break
                     except:
                         self.response.close(True)
@@ -474,7 +644,7 @@ class ClientRequest:
                         except Exception as err:
                             exc = err
                     elif isinstance(result, (bytes, bytearray)):
-                        yield from request.write(result)
+                        yield from request.write(result, drain=True)
                         value = None
                     else:
                         raise ValueError(
@@ -484,7 +654,7 @@ class ClientRequest:
             elif isinstance(self.body, asyncio.StreamReader):
                 chunk = yield from self.body.read(streams.DEFAULT_LIMIT)
                 while chunk:
-                    yield from request.write(chunk)
+                    yield from request.write(chunk, drain=True)
                     chunk = yield from self.body.read(streams.DEFAULT_LIMIT)
 
             elif isinstance(self.body, streams.DataQueue):
@@ -493,7 +663,7 @@ class ClientRequest:
                         chunk = yield from self.body.read()
                         if chunk is EOF_MARKER:
                             break
-                        yield from request.write(chunk)
+                        yield from request.write(chunk, drain=True)
                     except streams.EofStream:
                         break
 
@@ -542,6 +712,11 @@ class ClientRequest:
             request.enable_chunked_encoding()
             request.add_chunking_filter(self.chunked)
 
+        # set default content-type
+        if (self.method in self.POST_METHODS and
+                hdrs.CONTENT_TYPE not in self.headers):
+            self.headers[hdrs.CONTENT_TYPE] = 'application/octet-stream'
+
         request.add_headers(
             *((k, v)
               for k, v in ((k, value)
@@ -666,7 +841,6 @@ class ClientResponse:
                 except http.cookies.CookieError as exc:
                     client_logger.warning(
                         'Can not load response cookies: %s', exc)
-            connection.share_cookies(self.cookies)
         return self
 
     def close(self, force=False):
@@ -678,6 +852,9 @@ class ClientResponse:
                 self.connection.close()
             else:
                 self.connection.release()
+                if self._reader is not None:
+                    self._reader.unset_parser()
+
             self.connection = None
             self._connection_wr = None
         if self._writer is not None and not self._writer.done():
@@ -740,7 +917,7 @@ class ClientResponse:
 
         if not encoding:
             encoding = params.get('charset')
-            if not encoding and chardet:
+            if not encoding:
                 encoding = chardet.detect(self._content)['encoding']
             if not encoding:
                 encoding = 'utf-8'
@@ -753,7 +930,10 @@ class ClientResponse:
         if self._content is None:
             yield from self.read()
 
-        return self._content.decode(self._get_encoding(encoding))
+        if encoding is None:
+            encoding = self._get_encoding(encoding)
+
+        return self._content.decode(encoding)
 
     @asyncio.coroutine
     def json(self, *, encoding=None, loads=json.loads):
@@ -769,4 +949,7 @@ class ClientResponse:
         if not self._content.strip():
             return None
 
-        return loads(self._content.decode(self._get_encoding(encoding)))
+        if encoding is None:
+            encoding = self._get_encoding(encoding)
+
+        return loads(self._content.decode(encoding))
