@@ -7,15 +7,11 @@ import io
 import unittest
 import unittest.mock
 import urllib.parse
+import os.path
 
 import aiohttp
 from aiohttp.client import ClientRequest, ClientResponse
 
-try:
-    import chardet
-except ImportError:  # pragma: no cover
-    chardet = None
-
 
 class ClientResponseTests(unittest.TestCase):
 
@@ -65,11 +61,8 @@ class ClientResponseTests(unittest.TestCase):
 
     def test_read_and_release_connection(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result(b'payload')
-            content.read.side_effect = second_call
             return fut
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
@@ -123,11 +116,8 @@ class ClientResponseTests(unittest.TestCase):
 
     def test_text(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {
             'CONTENT-TYPE': 'application/json;charset=cp1251'}
@@ -141,31 +131,26 @@ class ClientResponseTests(unittest.TestCase):
 
     def test_text_custom_encoding(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {
             'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
         self.response.close = unittest.mock.Mock()
+        self.response._get_encoding = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.text(encoding='cp1251'))
         self.assertEqual(res, '{"тест": "пройден"}')
         self.assertTrue(self.response.close.called)
+        self.assertFalse(self.response._get_encoding.called)
 
-    @unittest.skipIf(chardet is None, "no chardet")
     def test_text_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
@@ -176,31 +161,10 @@ class ClientResponseTests(unittest.TestCase):
         self.assertEqual(res, '{"тест": "пройден"}')
         self.assertTrue(self.response.close.called)
 
-    def test_text_detect_encoding_without_chardet(self):
-        def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
-            fut = asyncio.Future(loop=self.loop)
-            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
-            return fut
-        self.response.headers = {'CONTENT-TYPE': 'application/json'}
-        content = self.response.content = unittest.mock.Mock()
-        content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
-
-        with unittest.mock.patch('aiohttp.client.chardet', None):
-            self.assertRaises(UnicodeDecodeError,
-                              self.loop.run_until_complete,
-                              self.response.text())
-
     def test_json(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {
             'CONTENT-TYPE': 'application/json;charset=cp1251'}
@@ -238,31 +202,26 @@ class ClientResponseTests(unittest.TestCase):
 
     def test_json_override_encoding(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {
             'CONTENT-TYPE': 'application/json;charset=utf8'}
         content = self.response.content = unittest.mock.Mock()
         content.read.side_effect = side_effect
         self.response.close = unittest.mock.Mock()
+        self.response._get_encoding = unittest.mock.Mock()
 
         res = self.loop.run_until_complete(
             self.response.json(encoding='cp1251'))
         self.assertEqual(res, {'тест': 'пройден'})
         self.assertTrue(self.response.close.called)
+        self.assertFalse(self.response._get_encoding.called)
 
-    @unittest.skipIf(chardet is None, "no chardet")
     def test_json_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
             fut = asyncio.Future(loop=self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
             return fut
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
         content = self.response.content = unittest.mock.Mock()
@@ -273,24 +232,6 @@ class ClientResponseTests(unittest.TestCase):
         self.assertEqual(res, {'тест': 'пройден'})
         self.assertTrue(self.response.close.called)
 
-    def test_json_detect_encoding_without_chardet(self):
-        def side_effect(*args, **kwargs):
-            def second_call(*args, **kwargs):
-                raise aiohttp.EofStream
-            fut = asyncio.Future(loop=self.loop)
-            fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
-            content.read.side_effect = second_call
-            return fut
-        self.response.headers = {'CONTENT-TYPE': 'application/json'}
-        content = self.response.content = unittest.mock.Mock()
-        content.read.side_effect = side_effect
-        self.response.close = unittest.mock.Mock()
-
-        with unittest.mock.patch('aiohttp.client.chardet', None):
-            self.assertRaises(UnicodeDecodeError,
-                              self.loop.run_until_complete,
-                              self.response.json())
-
     def test_override_flow_control(self):
         class MyResponse(ClientResponse):
             flow_control_class = aiohttp.FlowControlDataQueue
@@ -300,6 +241,13 @@ class ClientResponseTests(unittest.TestCase):
         with self.assertWarns(ResourceWarning):
             del response
 
+    @unittest.mock.patch('aiohttp.client.chardet')
+    def test_get_encoding_unknown(self, m_chardet):
+        m_chardet.detect.return_value = {'encoding': None}
+
+        self.response.headers = {'CONTENT-TYPE': 'application/json'}
+        self.assertEqual(self.response._get_encoding(None), 'utf-8')
+
 
 class ClientRequestTests(unittest.TestCase):
 
@@ -606,6 +554,39 @@ class ClientRequestTests(unittest.TestCase):
         self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
         self.assertNotIn('CONTENT-LENGTH', req.headers)
 
+    def test_file_upload_not_chunked(self):
+        here = os.path.dirname(__file__)
+        fname = os.path.join(here, 'sample.key')
+        with open(fname, 'rb') as f:
+            req = ClientRequest(
+                'post', 'http://python.org/',
+                data=f)
+            self.assertFalse(req.chunked)
+            self.assertEqual(req.headers['CONTENT-LENGTH'],
+                             str(os.path.getsize(fname)))
+
+    def test_file_upload_not_chunked_seek(self):
+        here = os.path.dirname(__file__)
+        fname = os.path.join(here, 'sample.key')
+        with open(fname, 'rb') as f:
+            f.seek(100)
+            req = ClientRequest(
+                'post', 'http://python.org/',
+                data=f)
+            self.assertEqual(req.headers['CONTENT-LENGTH'],
+                             str(os.path.getsize(fname) - 100))
+
+    def test_file_upload_force_chunked(self):
+        here = os.path.dirname(__file__)
+        fname = os.path.join(here, 'sample.key')
+        with open(fname, 'rb') as f:
+            req = ClientRequest(
+                'post', 'http://python.org/',
+                data=f,
+                chunked=True)
+            self.assertTrue(req.chunked)
+            self.assertNotIn('CONTENT-LENGTH', req.headers)
+
     def test_expect100(self):
         req = ClientRequest('get', 'http://python.org/',
                             expect100=True, loop=self.loop)
@@ -664,7 +645,6 @@ class ClientRequestTests(unittest.TestCase):
         def gen():
             yield b'binary data'
             yield from fut
-            return b' result'
 
         req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
@@ -689,7 +669,6 @@ class ClientRequestTests(unittest.TestCase):
         @asyncio.coroutine
         def gen():
             yield object()
-            return b' result'
 
         req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
@@ -702,7 +681,6 @@ class ClientRequestTests(unittest.TestCase):
 
         def gen():
             yield from fut
-            return b' result'
 
         req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
