@@ -66,27 +66,6 @@ class HttpConnectionTests(unittest.TestCase):
         self.assertIsNone(conn._transport)
         self.assertFalse(self.connector._release.called)
 
-    def test_no_share_cookies(self):
-        connector = aiohttp.BaseConnector(share_cookies=False, loop=self.loop)
-
-        conn = Connection(
-            connector, self.key, self.request,
-            self.transport, self.protocol, self.loop)
-        self.assertEqual(connector.cookies, {})
-        conn.share_cookies({'c1': 'cookie1'})
-        self.assertEqual(connector.cookies, {})
-
-    def test_share_cookies(self):
-        connector = aiohttp.BaseConnector(share_cookies=True, loop=self.loop)
-
-        conn = Connection(
-            connector, self.key, self.request,
-            self.transport, self.protocol, self.loop)
-        self.assertEqual(connector.cookies, {})
-        conn.share_cookies({'c1': 'cookie1'})
-        self.assertEqual(connector.cookies,
-                         http.cookies.SimpleCookie({'c1': 'cookie1'}))
-
 
 class BaseConnectorTests(unittest.TestCase):
 
@@ -407,6 +386,11 @@ class HttpClientConnectorTests(unittest.TestCase):
             self.assertEqual(r.status, 200)
             r.close()
 
+    def test_connector_cookie_deprecation(self):
+        with self.assertWarnsRegex(DeprecationWarning,
+                                   "^Using `share_cookies` is deprecated"):
+            aiohttp.TCPConnector(share_cookies=True, loop=self.loop)
+
 
 class ProxyConnectorTests(unittest.TestCase):
 
@@ -467,7 +451,7 @@ class ProxyConnectorTests(unittest.TestCase):
         ClientRequestMock.assert_called_with(
             'GET', 'http://proxy.example.com',
             auth=None,
-            headers={'Host': 'www.python.org'},
+            headers={'HOST': 'www.python.org'},
             loop=loop_mock)
 
     def test_proxy_auth(self):
@@ -606,6 +590,7 @@ class ProxyConnectorTests(unittest.TestCase):
         req = ClientRequest('GET', 'https://www.python.org')
         self.loop.run_until_complete(connector._create_connection(req))
 
+        self.assertEqual(req.path, '/')
         self.assertEqual(proxy_req.method, 'CONNECT')
         self.assertEqual(proxy_req.path, 'www.python.org:443')
         tr.pause_reading.assert_called_once_with()
