@@ -21,27 +21,14 @@ class StreamParserTests(unittest.TestCase):
         self.loop.close()
 
     def test_at_eof(self):
-        proto = parsers.StreamParser()
+        proto = parsers.StreamParser(loop=self.loop)
         self.assertFalse(proto.at_eof())
 
         proto.feed_eof()
         self.assertTrue(proto.at_eof())
 
-    def test_resume_stream(self):
-        transp = unittest.mock.Mock()
-
-        proto = parsers.StreamParser()
-        proto.set_transport(transp)
-        proto._paused = True
-        proto._stream_paused = True
-        proto.resume_stream()
-
-        transp.resume_reading.assert_called_with()
-        self.assertFalse(proto._paused)
-        self.assertFalse(proto._stream_paused)
-
     def test_exception(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         self.assertIsNone(stream.exception())
 
         exc = ValueError()
@@ -49,7 +36,7 @@ class StreamParserTests(unittest.TestCase):
         self.assertIs(stream.exception(), exc)
 
     def test_exception_connection_error(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         self.assertIsNone(stream.exception())
 
         exc = ConnectionError()
@@ -60,7 +47,7 @@ class StreamParserTests(unittest.TestCase):
         self.assertIs(stream.exception().__context__, exc)
 
     def test_exception_waiter(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
 
         stream._parser = self.lines_parser
         buf = stream._output = parsers.FlowControlDataQueue(
@@ -71,38 +58,19 @@ class StreamParserTests(unittest.TestCase):
         self.assertIs(buf.exception(), exc)
 
     def test_feed_data(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
 
         stream.feed_data(self.DATA)
         self.assertEqual(self.DATA, bytes(stream._buffer))
 
     def test_feed_none_data(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
 
         stream.feed_data(None)
         self.assertEqual(b'', bytes(stream._buffer))
 
-    def test_feed_data_pause_reading(self):
-        transp = unittest.mock.Mock()
-
-        proto = parsers.StreamParser()
-        proto.set_transport(transp)
-        proto.feed_data(b'1' * (2 ** 16 * 3))
-        transp.pause_reading.assert_called_with()
-        self.assertTrue(proto._paused)
-
-    def test_feed_data_pause_reading_not_supported(self):
-        transp = unittest.mock.Mock()
-
-        proto = parsers.StreamParser()
-        proto.set_transport(transp)
-
-        transp.pause_reading.side_effect = NotImplementedError()
-        proto.feed_data(b'1' * (2 ** 16 * 3))
-        self.assertIsNone(proto._transport)
-
     def test_set_parser_unset_prev(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.set_parser(self.lines_parser)
 
         unset = stream.unset_parser = unittest.mock.Mock()
@@ -111,7 +79,7 @@ class StreamParserTests(unittest.TestCase):
         self.assertTrue(unset.called)
 
     def test_set_parser_exception(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
 
         exc = ValueError()
         stream.set_exception(exc)
@@ -119,13 +87,14 @@ class StreamParserTests(unittest.TestCase):
         self.assertIs(s.exception(), exc)
 
     def test_set_parser_feed_existing(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         stream.feed_data(b'\r\nline2\r\ndata')
         s = stream.set_parser(self.lines_parser)
 
-        self.assertEqual([bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
-                         list(s._buffer))
+        self.assertEqual(
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
+            list(s._buffer))
         self.assertEqual(b'data', bytes(stream._buffer))
         self.assertIsNotNone(stream._parser)
 
@@ -139,20 +108,21 @@ class StreamParserTests(unittest.TestCase):
             yield from buf.read(1)
             raise ValueError()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         s = stream.set_parser(p)
         self.assertIsInstance(s.exception(), ValueError)
 
     def test_set_parser_feed_existing_eof(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         stream.feed_data(b'\r\nline2\r\ndata')
         stream.feed_eof()
         s = stream.set_parser(self.lines_parser)
 
-        self.assertEqual([bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
-                         list(s._buffer))
+        self.assertEqual(
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
+            list(s._buffer))
         self.assertEqual(b'data', bytes(stream._buffer))
         self.assertIsNone(stream._parser)
 
@@ -164,7 +134,7 @@ class StreamParserTests(unittest.TestCase):
             except parsers.EofStream:
                 raise ValueError()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         stream.feed_eof()
         s = stream.set_parser(p)
@@ -175,7 +145,7 @@ class StreamParserTests(unittest.TestCase):
             while True:
                 yield  # read chunk
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         stream.feed_eof()
         s = stream.set_parser(p)
@@ -183,12 +153,12 @@ class StreamParserTests(unittest.TestCase):
         self.assertIsInstance(s.exception(), RuntimeError)
 
     def test_set_parser_unset(self):
-        stream = parsers.StreamParser(paused=False)
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(self.lines_parser)
 
         stream.feed_data(b'line1\r\nline2\r\n')
         self.assertEqual(
-            [bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
             list(s._buffer))
         self.assertEqual(b'', bytes(stream._buffer))
         stream.unset_parser()
@@ -198,23 +168,27 @@ class StreamParserTests(unittest.TestCase):
     def test_set_parser_feed_existing_stop(self):
         def LinesParser(out, buf):
             try:
-                out.feed_data((yield from buf.readuntil(b'\n')))
-                out.feed_data((yield from buf.readuntil(b'\n')))
+                chunk = yield from buf.readuntil(b'\n')
+                out.feed_data(chunk, len(chunk))
+
+                chunk = yield from buf.readuntil(b'\n')
+                out.feed_data(chunk, len(chunk))
             finally:
                 out.feed_eof()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         stream.feed_data(b'line1')
         stream.feed_data(b'\r\nline2\r\ndata')
         s = stream.set_parser(LinesParser)
 
-        self.assertEqual(b'line1\r\nline2\r\n', b''.join(s._buffer))
+        self.assertEqual(
+            b'line1\r\nline2\r\n', b''.join(d for d, _ in s._buffer))
         self.assertEqual(b'data', bytes(stream._buffer))
         self.assertIsNone(stream._parser)
         self.assertTrue(s._eof)
 
     def test_feed_parser(self):
-        stream = parsers.StreamParser(paused=False)
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(self.lines_parser)
 
         stream.feed_data(b'line1')
@@ -222,8 +196,9 @@ class StreamParserTests(unittest.TestCase):
         self.assertEqual(b'data', bytes(stream._buffer))
 
         stream.feed_eof()
-        self.assertEqual([bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
-                         list(s._buffer))
+        self.assertEqual(
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
+            list(s._buffer))
         self.assertEqual(b'data', bytes(stream._buffer))
         self.assertTrue(s.is_eof())
 
@@ -232,7 +207,7 @@ class StreamParserTests(unittest.TestCase):
             yield  # read chunk
             raise ValueError()
 
-        stream = parsers.StreamParser(paused=False)
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -243,7 +218,7 @@ class StreamParserTests(unittest.TestCase):
         def p(out, buf):
             yield  # chunk
 
-        stream = parsers.StreamParser(paused=False)
+        stream = parsers.StreamParser(loop=self.loop)
         stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -258,7 +233,7 @@ class StreamParserTests(unittest.TestCase):
             except parsers.EofStream:
                 raise ValueError()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -275,7 +250,7 @@ class StreamParserTests(unittest.TestCase):
             except parsers.EofStream:
                 out.feed_eof()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -287,7 +262,7 @@ class StreamParserTests(unittest.TestCase):
             while True:
                 yield  # read chunk
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -296,13 +271,13 @@ class StreamParserTests(unittest.TestCase):
         self.assertIsInstance(s.exception(), RuntimeError)
 
     def test_feed_parser2(self):
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(self.lines_parser)
 
         stream.feed_data(b'line1\r\nline2\r\n')
         stream.feed_eof()
         self.assertEqual(
-            [bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
             list(s._buffer))
         self.assertEqual(b'', bytes(stream._buffer))
         self.assertTrue(s._eof)
@@ -315,7 +290,7 @@ class StreamParserTests(unittest.TestCase):
             except parsers.EofStream:
                 raise ValueError()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -328,7 +303,7 @@ class StreamParserTests(unittest.TestCase):
             while True:
                 yield  # read chunk
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -344,7 +319,7 @@ class StreamParserTests(unittest.TestCase):
             except parsers.EofStream:
                 out.feed_eof()
 
-        stream = parsers.StreamParser()
+        stream = parsers.StreamParser(loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_data(b'line1')
@@ -359,7 +334,8 @@ class StreamParserTests(unittest.TestCase):
         class CustomEofErr(Exception):
             pass
 
-        stream = parsers.StreamParser(eof_exc_class=CustomEofErr)
+        stream = parsers.StreamParser(
+            eof_exc_class=CustomEofErr, loop=self.loop)
         s = stream.set_parser(p)
 
         stream.feed_eof()
@@ -368,17 +344,24 @@ class StreamParserTests(unittest.TestCase):
 
 class StreamProtocolTests(unittest.TestCase):
 
+    def setUp(self):
+        self.loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(None)
+
+    def tearDown(self):
+        self.loop.close()
+
     def test_connection_made(self):
         tr = unittest.mock.Mock()
 
-        proto = parsers.StreamProtocol()
+        proto = parsers.StreamProtocol(loop=self.loop)
         self.assertIsNone(proto.transport)
 
         proto.connection_made(tr)
         self.assertIs(proto.transport, tr)
 
     def test_connection_lost(self):
-        proto = parsers.StreamProtocol()
+        proto = parsers.StreamProtocol(loop=self.loop)
         proto.connection_made(unittest.mock.Mock())
         proto.connection_lost(None)
         self.assertIsNone(proto.transport)
@@ -386,7 +369,7 @@ class StreamProtocolTests(unittest.TestCase):
         self.assertTrue(proto.reader._eof)
 
     def test_connection_lost_exc(self):
-        proto = parsers.StreamProtocol()
+        proto = parsers.StreamProtocol(loop=self.loop)
         proto.connection_made(unittest.mock.Mock())
 
         exc = ValueError()
@@ -394,25 +377,13 @@ class StreamProtocolTests(unittest.TestCase):
         self.assertIs(proto.reader.exception(), exc)
 
     def test_data_received(self):
-        proto = parsers.StreamProtocol()
+        proto = parsers.StreamProtocol(loop=self.loop)
         proto.connection_made(unittest.mock.Mock())
         proto.reader = unittest.mock.Mock()
 
         proto.data_received(b'data')
         proto.reader.feed_data.assert_called_with(b'data')
 
-    def test_drain_waiter(self):
-        proto = parsers.StreamProtocol(loop=unittest.mock.Mock())
-        proto._paused = False
-        self.assertEqual(proto._make_drain_waiter(), ())
-
-        proto._paused = True
-        fut = proto._make_drain_waiter()
-        self.assertIsInstance(fut, asyncio.Future)
-
-        fut2 = proto._make_drain_waiter()
-        self.assertIs(fut, fut2)
-
 
 class ParserBufferTests(unittest.TestCase):
 
@@ -597,7 +568,7 @@ class ParserBufferTests(unittest.TestCase):
             p.send(d)
 
         self.assertEqual(
-            [bytearray(b'line1\r\n'), bytearray(b'line2\r\n')],
+            [(bytearray(b'line1\r\n'), 7), (bytearray(b'line2\r\n'), 7)],
             list(out._buffer))
         try:
             p.throw(parsers.EofStream())
@@ -616,7 +587,8 @@ class ParserBufferTests(unittest.TestCase):
             p.send(d)
 
         self.assertEqual(
-            [bytearray(b'line1'), bytearray(b'line2')], list(out._buffer))
+            [(bytearray(b'line1'), 5), (bytearray(b'line2'), 5)],
+            list(out._buffer))
         try:
             p.throw(parsers.EofStream())
         except StopIteration:
