@@ -3,10 +3,14 @@ import os
 import unittest
 from unittest import mock
 import aiohttp.web
+from aiohttp import hdrs
 from aiohttp.web import (UrlDispatcher, Request, Response,
                          HTTPMethodNotAllowed, HTTPNotFound)
 from aiohttp.multidict import CIMultiDict
 from aiohttp.protocol import HttpVersion, RawRequestMessage
+from aiohttp.web_urldispatcher import (_defaultExpectHandler,
+                                       DynamicRoute,
+                                       PlainRoute)
 
 
 class TestUrlDispatcher(unittest.TestCase):
@@ -39,6 +43,28 @@ class TestUrlDispatcher(unittest.TestCase):
 
         return handler
 
+    def test_register_route(self):
+        handler = self.make_handler()
+        route = PlainRoute('GET', handler, 'test', '/handler/to/path')
+        self.router.register_route(route)
+
+        req = self.make_request('GET', '/handler/to/path')
+        info = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertIsNotNone(info)
+        self.assertEqual(0, len(info))
+        self.assertIs(route, info.route)
+        self.assertIs(handler, info.handler)
+        self.assertEqual(info.route.name, 'test')
+
+    def test_register_route_checks(self):
+        self.assertRaises(
+            AssertionError, self.router.register_route, object())
+
+        handler = self.make_handler()
+        route = PlainRoute('GET', handler, 'test', '/handler/to/path')
+        self.router.register_route(route)
+        self.assertRaises(ValueError, self.router.register_route, route)
+
     def test_add_route_root(self):
         handler = self.make_handler()
         self.router.add_route('GET', '/', handler)
@@ -112,11 +138,6 @@ class TestUrlDispatcher(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.router.add_route('post', '/post/{id"}', handler)
 
-    def test_add_url_invalid5(self):
-        handler = self.make_handler()
-        with self.assertRaises(ValueError):
-            self.router.add_route('post', '/post"{id}', handler)
-
     def test_add_url_escaping(self):
         handler = self.make_handler()
         self.router.add_route('GET', '/+$', handler)
@@ -126,6 +147,21 @@ class TestUrlDispatcher(unittest.TestCase):
         self.assertIsNotNone(info)
         self.assertIs(handler, info.handler)
 
+    def test_any_method(self):
+        handler = self.make_handler()
+        route = self.router.add_route(hdrs.METH_ANY, '/', handler)
+
+        req = self.make_request('GET', '/')
+        info1 = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertIsNotNone(info1)
+        self.assertIs(route, info1.route)
+
+        req = self.make_request('POST', '/')
+        info2 = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertIsNotNone(info2)
+
+        self.assertIs(info1.route, info2.route)
+
     def test_match_second_result_in_table(self):
         handler1 = self.make_handler()
         handler2 = self.make_handler()
@@ -298,6 +334,25 @@ class TestUrlDispatcher(unittest.TestCase):
         self.assertIsNotNone(info)
         self.assertEqual({'to': '1234'}, info)
 
+        self.router.add_route('GET', r'/handler/{name}.html', handler)
+        req = self.make_request('GET', '/handler/test.html')
+        info = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertEqual({'name': 'test'}, info)
+
+    def test_add_route_with_re_and_slashes(self):
+        handler = self.make_handler()
+        self.router.add_route('GET', r'/handler/{to:[^/]+/?}', handler)
+        req = self.make_request('GET', '/handler/1234/')
+        info = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertIsNotNone(info)
+        self.assertEqual({'to': '1234/'}, info)
+
+        self.router.add_route('GET', r'/handler/{to:.+}', handler)
+        req = self.make_request('GET', '/handler/1234/5/6/7')
+        info = self.loop.run_until_complete(self.router.resolve(req))
+        self.assertIsNotNone(info)
+        self.assertEqual({'to': '1234/5/6/7'}, info)
+
     def test_add_route_with_re_not_match(self):
         handler = self.make_handler()
         self.router.add_route('GET', r'/handler/{to:\d+}', handler)
@@ -312,7 +367,6 @@ class TestUrlDispatcher(unittest.TestCase):
     def test_add_route_with_re_including_slashes(self):
         handler = self.make_handler()
         self.router.add_route('GET', r'/handler/{to:.+}/tail', handler)
-
         req = self.make_request('GET', '/handler/re/with/slashes/tail')
         info = self.loop.run_until_complete(self.router.resolve(req))
         self.assertIsNotNone(info)
@@ -322,9 +376,9 @@ class TestUrlDispatcher(unittest.TestCase):
         handler = self.make_handler()
         with self.assertRaises(ValueError) as ctx:
             self.router.add_route('GET', r'/handler/{to:+++}', handler)
-        self.assertEqual(
-            "Bad pattern '/handler/(?P<to>+++)': nothing to repeat",
-            str(ctx.exception))
+        s = str(ctx.exception)
+        self.assertTrue(s.startswith(
+            "Bad pattern '\/handler\/(?P<to>+++)': nothing to repeat"), s)
         self.assertIsNone(ctx.exception.__cause__)
 
     def test_route_dynamic_with_regex_spec(self):
@@ -334,3 +388,133 @@ class TestUrlDispatcher(unittest.TestCase):
 
         url = route.url(parts={'num': '123'})
         self.assertEqual('/get/123', url)
+
+    def test_route_dynamic_with_regex_spec_and_trailing_slash(self):
+        handler = self.make_handler()
+        route = self.router.add_route('GET', '/get/{num:^\d+}/', handler,
+                                      name='name')
+
+        url = route.url(parts={'num': '123'})
+        self.assertEqual('/get/123/', url)
+
+    def test_route_dynamic_with_regex(self):
+        handler = self.make_handler()
+        route = self.router.add_route('GET', r'/{one}/{two:.+}', handler)
+
+        url = route.url(parts={'one': 1, 'two': 2})
+        self.assertEqual('/1/2', url)
+
+    def test_regular_match_info(self):
+
+        @asyncio.coroutine
+        def go():
+            handler = self.make_handler()
+            self.router.add_route('GET', '/get/{name}', handler)
+
+            req = self.make_request('GET', '/get/john')
+            match_info = yield from self.router.resolve(req)
+            self.maxDiff = None
+            self.assertRegex(repr(match_info),
+                             "<MatchInfo {'name': 'john'}: <DynamicRoute.+>>")
+
+        self.loop.run_until_complete(go())
+
+    def test_not_found_repr(self):
+
+        @asyncio.coroutine
+        def go():
+            req = self.make_request('POST', '/path/to')
+            match_info = yield from self.router.resolve(req)
+            self.assertEqual("<MatchInfo: not found>", repr(match_info))
+
+        self.loop.run_until_complete(go())
+
+    def test_not_allowed_repr(self):
+
+        @asyncio.coroutine
+        def go():
+            handler = self.make_handler()
+            self.router.add_route('GET', '/path/to', handler)
+
+            handler2 = self.make_handler()
+            self.router.add_route('POST', '/path/to', handler2)
+
+            req = self.make_request('PUT', '/path/to')
+            match_info = yield from self.router.resolve(req)
+            self.assertEqual("<MatchInfo: method PUT is not allowed "
+                             "(allowed methods: GET, POST>", repr(match_info))
+
+        self.loop.run_until_complete(go())
+
+    def test_default_expect_handler(self):
+        route = self.router.add_route('GET', '/', self.make_handler())
+        self.assertIs(route._expect_handler, _defaultExpectHandler)
+
+    def test_custom_expect_handler_plain(self):
+
+        @asyncio.coroutine
+        def handler(request):
+            pass
+
+        route = self.router.add_route(
+            'GET', '/', self.make_handler(), expect_handler=handler)
+        self.assertIs(route._expect_handler, handler)
+        self.assertIsInstance(route, PlainRoute)
+
+    def test_custom_expect_handler_dynamic(self):
+
+        @asyncio.coroutine
+        def handler(request):
+            pass
+
+        route = self.router.add_route(
+            'GET', '/get/{name}', self.make_handler(), expect_handler=handler)
+        self.assertIs(route._expect_handler, handler)
+        self.assertIsInstance(route, DynamicRoute)
+
+    def test_expect_handler_non_coroutine(self):
+
+        def handler(request):
+            pass
+
+        self.assertRaises(
+            AssertionError, self.router.add_route,
+            'GET', '/', self.make_handler(), expect_handler=handler)
+
+    def test_dynamic_match_non_ascii(self):
+
+        @asyncio.coroutine
+        def go():
+            handler = self.make_handler()
+            self.router.add_route('GET', '/{var}', handler)
+            req = self.make_request(
+                'GET',
+                '/%D1%80%D1%83%D1%81%20%D1%82%D0%B5%D0%BA%D1%81%D1%82')
+            match_info = yield from self.router.resolve(req)
+            self.assertEqual({'var': 'рус текст'}, match_info)
+
+        self.loop.run_until_complete(go())
+
+    def test_dynamic_match_with_static_part(self):
+
+        @asyncio.coroutine
+        def go():
+            handler = self.make_handler()
+            self.router.add_route('GET', '/{name}.html', handler)
+            req = self.make_request('GET', '/file.html')
+            match_info = yield from self.router.resolve(req)
+            self.assertEqual({'name': 'file'}, match_info)
+
+        self.loop.run_until_complete(go())
+
+    def test_dynamic_match_two_part2(self):
+
+        @asyncio.coroutine
+        def go():
+            handler = self.make_handler()
+            self.router.add_route('GET', '/{name}.{ext}', handler)
+            req = self.make_request('GET', '/file.html')
+            match_info = yield from self.router.resolve(req)
+            self.assertEqual({'name': 'file', 'ext': 'html'}, match_info)
+
+        self.loop.run_until_complete(go())
