@@ -7,10 +7,13 @@ from aiohttp import web, request
 class TestWebFunctional(unittest.TestCase):
 
     def setUp(self):
+        self.handler = None
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(None)
 
     def tearDown(self):
+        if self.handler:
+            self.loop.run_until_complete(self.handler.finish_connections())
         self.loop.close()
 
     def find_unused_port(self):
@@ -26,8 +29,10 @@ class TestWebFunctional(unittest.TestCase):
         app.router.add_route(method, path, handler)
 
         port = self.find_unused_port()
-        srv = yield from self.loop.create_server(
-            app.make_handler(debug=True), '127.0.0.1', port)
+        self.handler = app.make_handler(debug=True)
+        srv = yield from self.loop.create_server(self.handler, '127.0.0.1',
+                                                 port)
+
         url = "http://127.0.0.1:{}".format(port) + path
         self.addCleanup(srv.close)
         return app, srv, url
@@ -56,7 +61,6 @@ class TestWebFunctional(unittest.TestCase):
             self.assertEqual(201, resp.status)
             txt = yield from resp.text()
             self.assertEqual('OK[MIDDLEWARE]', txt)
-
         self.loop.run_until_complete(go())
 
     def test_middleware_handles_exception(self):
