@@ -18,6 +18,8 @@ class TestWebRequest(unittest.TestCase):
 
     def make_request(self, method, path, headers=CIMultiDict(), *,
                      version=HttpVersion(1, 1), closing=False):
+        if version < HttpVersion(1, 1):
+            closing = True
         self.app = mock.Mock()
         message = RawRequestMessage(method, path, version, headers, closing,
                                     False)
@@ -169,3 +171,7 @@ class TestWebRequest(unittest.TestCase):
         self.assertTrue(isinstance(req, dict))
         req['key'] = 'value'
         self.assertEqual('value', req['key'])
+
+    def test___repr__(self):
+        req = self.make_request('GET', '/path/to')
+        self.assertEqual("<Request GET /path/to >", repr(req))
